@@ -1,5 +1,8 @@
 use anyhow::Result;
-use probe_rs::Probe;
+use probe_rs::{
+    architecture::arm::{sequences::DefaultArmSequence, ApAddress, DpAddress},
+    Probe,
+};
 
 fn main() -> Result<()> {
     pretty_env_logger::init();
@@ -11,12 +14,17 @@ fn main() -> Result<()> {
     let mut probe = probes[0].open()?;
 
     probe.attach_to_unspecified()?;
-    let mut iface = probe.try_into_arm_interface().unwrap();
+    let iface = probe.try_into_arm_interface().unwrap();
+
+    let mut iface = iface.initialize(DefaultArmSequence::create())?;
 
     // This is an example on how to do a "recover" operation (erase+unlock a locked chip)
     // on an nRF52840 target.
 
-    let port = 1;
+    let port = ApAddress {
+        dp: DpAddress::Default,
+        ap: 1,
+    };
 
     const RESET: u8 = 0;
     const ERASEALL: u8 = 4;
