@@ -1,7 +1,11 @@
 use super::super::ap::{
     AccessPortError, AddressIncrement, ApAccess, ApRegister, DataSize, MemoryAp, CSW, DRW, TAR,
 };
-use crate::architecture::arm::{dp::DpAccess, MemoryApInformation};
+use crate::architecture::arm::communication_interface::SwdSequence;
+use crate::architecture::arm::ArmCommunicationInterface;
+use crate::architecture::arm::{
+    communication_interface::Initialized, dp::DpAccess, MemoryApInformation,
+};
 use crate::{CommunicationInterface, CoreRegister, CoreRegisterAddress, DebugProbeError, Error};
 use scroll::{Pread, Pwrite, LE};
 use std::convert::TryInto;
@@ -12,7 +16,7 @@ use std::{
 
 use bitfield::bitfield;
 
-pub trait ArmProbe {
+pub trait ArmProbe: SwdSequence {
     fn read_core_reg(&mut self, ap: MemoryAp, addr: CoreRegisterAddress) -> Result<u32, Error>;
     fn write_core_reg(
         &mut self,
@@ -28,6 +32,10 @@ pub trait ArmProbe {
     fn write_32(&mut self, ap: MemoryAp, address: u32, data: &[u32]) -> Result<(), Error>;
 
     fn flush(&mut self) -> Result<(), Error>;
+
+    fn get_arm_communication_interface(
+        &mut self,
+    ) -> Result<&mut ArmCommunicationInterface<Initialized>, Error>;
 }
 
 /// A struct to give access to a targets memory using a certain DAP.
@@ -311,7 +319,9 @@ where
         )?;
 
         remaining_data_len -= first_chunk_size_words;
-        address += (4 * first_chunk_size_words) as u32;
+        address = address
+            .checked_add((4 * first_chunk_size_words) as u32)
+            .ok_or(AccessPortError::OutOfBoundsError)?;
         data_offset += first_chunk_size_words;
 
         while remaining_data_len > 0 {
@@ -569,7 +579,7 @@ where
 
         // Copy input data into buffer at the correct location
         let start = (address - aligned.start) as usize;
-        buf8[start..start + data.len()].copy_from_slice(&data);
+        buf8[start..start + data.len()].copy_from_slice(data);
 
         // Convert buffer to 32-bit words
         let mut buf32 = vec![0u32; aligned.len() / 4];
@@ -584,6 +594,21 @@ where
     }
 }
 
+impl<AP> SwdSequence for ADIMemoryInterface<'_, AP>
+where
+    AP: CommunicationInterface + ApAccess + DpAccess,
+{
+    fn swj_sequence(&mut self, bit_len: u8, bits: u64) -> Result<(), Error> {
+        self.get_arm_communication_interface()?
+            .swj_sequence(bit_len, bits)
+    }
+
+    fn swj_pins(&mut self, pin_out: u32, pin_select: u32, pin_wait: u32) -> Result<u32, Error> {
+        self.get_arm_communication_interface()?
+            .swj_pins(pin_out, pin_select, pin_wait)
+    }
+}
+
 impl<AP> ArmProbe for ADIMemoryInterface<'_, AP>
 where
     AP: CommunicationInterface + ApAccess + DpAccess,
@@ -668,6 +693,12 @@ where
 
         Ok(())
     }
+
+    fn get_arm_communication_interface(
+        &mut self,
+    ) -> Result<&mut ArmCommunicationInterface<Initialized>, Error> {
+        CommunicationInterface::get_arm_communication_interface(self.interface)
+    }
 }
 
 bitfield! {
@@ -782,22 +813,29 @@ fn aligned_range(address: u32, len: usize) -> Result<Range<u32>, AccessPortError
 
 #[cfg(test)]
 mod tests {
-    use crate::architecture::arm::MemoryApInformation;
+    use crate::architecture::arm::{ap::AccessPort, ApAddress, DpAddress, MemoryApInformation};
 
     use super::super::super::ap::memory_ap::mock::MockMemoryAp;
+    use super::super::super::ap::memory_ap::MemoryAp;
     use super::ADIMemoryInterface;
 
+    const DUMMY_AP: MemoryAp = MemoryAp::new(ApAddress {
+        dp: DpAddress::Default,
+        ap: 0,
+    });
+
     impl<'interface> ADIMemoryInterface<'interface, MockMemoryAp> {
         /// Creates a new MemoryInterface for given AccessPort.
         fn new_mock(
             mock: &'interface mut MockMemoryAp,
         ) -> ADIMemoryInterface<'interface, MockMemoryAp> {
             let ap_information = MemoryApInformation {
-                port_number: 0,
+                address: DUMMY_AP.ap_address(),
                 only_32bit_data_size: false,
-                debug_base_address: 0xf000_0000,
                 supports_hnonsec: false,
+                debug_base_address: 0xf000_0000,
             };
+
             Self::new(mock, &ap_information).unwrap()
         }
 
@@ -822,7 +860,7 @@ mod tests {
 
         for &address in &[0, 4] {
             let value = mi
-                .read_word_32(0.into(), address)
+                .read_word_32(DUMMY_AP, address)
                 .expect("read_word_32 failed");
             assert_eq!(value, DATA32[address as usize / 4]);
         }
@@ -836,7 +874,7 @@ mod tests {
 
         for address in 0..8 {
             let value = mi
-                .read_word_8(0.into(), address)
+                .read_word_8(DUMMY_AP, address)
                 .unwrap_or_else(|_| panic!("read_word_8 failed, address = {}", address));
             assert_eq!(value, DATA8[address as usize], "address = {}", address);
         }
@@ -851,7 +889,7 @@ mod tests {
             let mut expected = Vec::from(mi.mock_memory());
             expected[(address as usize)..(address as usize) + 4].copy_from_slice(&DATA8[..4]);
 
-            mi.write_word_32(0.into(), address, DATA32[0])
+            mi.write_word_32(DUMMY_AP, address, DATA32[0])
                 .unwrap_or_else(|_| panic!("write_word_32 failed, address = {}", address));
             assert_eq!(
                 mi.mock_memory(),
@@ -871,7 +909,7 @@ mod tests {
             let mut expected = Vec::from(mi.mock_memory());
             expected[address] = DATA8[0];
 
-            mi.write_word_8(0.into(), address as u32, DATA8[0])
+            mi.write_word_8(DUMMY_AP, address as u32, DATA8[0])
                 .unwrap_or_else(|_| panic!("write_word_8 failed, address = {}", address));
             assert_eq!(
                 mi.mock_memory(),
@@ -891,7 +929,7 @@ mod tests {
         for &address in &[0, 4] {
             for len in 0..3 {
                 let mut data = vec![0u32; len];
-                mi.read_32(0.into(), address, &mut data)
+                mi.read_32(DUMMY_AP, address, &mut data)
                     .unwrap_or_else(|_| {
                         panic!("read_32 failed, address = {}, len = {}", address, len)
                     });
@@ -913,7 +951,7 @@ mod tests {
         let mut mi = ADIMemoryInterface::new_mock(&mut mock);
 
         for &address in &[1, 3, 127] {
-            assert!(mi.read_32(0.into(), address, &mut [0u32; 4]).is_err());
+            assert!(mi.read_32(DUMMY_AP, address, &mut [0u32; 4]).is_err());
         }
     }
 
@@ -926,7 +964,7 @@ mod tests {
         for address in 0..4 {
             for len in 0..12 {
                 let mut data = vec![0u8; len];
-                mi.read_8(0.into(), address, &mut data).unwrap_or_else(|_| {
+                mi.read_8(DUMMY_AP, address, &mut data).unwrap_or_else(|_| {
                     panic!("read_8 failed, address = {}, len = {}", address, len)
                 });
 
@@ -953,7 +991,7 @@ mod tests {
                     .copy_from_slice(&DATA8[..len * 4]);
 
                 let data = &DATA32[..len];
-                mi.write_32(0.into(), address, data).unwrap_or_else(|_| {
+                mi.write_32(DUMMY_AP, address, data).unwrap_or_else(|_| {
                     panic!("write_32 failed, address = {}, len = {}", address, len)
                 });
 
@@ -975,7 +1013,7 @@ mod tests {
 
         for &address in &[1, 3, 127] {
             assert!(mi
-                .write_32(0.into(), address, &[0xDEAD_BEEF, 0xABBA_BABE])
+                .write_32(DUMMY_AP, address, &[0xDEAD_BEEF, 0xABBA_BABE])
                 .is_err());
         }
     }
@@ -991,7 +1029,7 @@ mod tests {
                 expected[address as usize..(address as usize) + len].copy_from_slice(&DATA8[..len]);
 
                 let data = &DATA8[..len];
-                mi.write_8(0.into(), address, data).unwrap_or_else(|_| {
+                mi.write_8(DUMMY_AP, address, data).unwrap_or_else(|_| {
                     panic!("write_8 failed, address = {}, len = {}", address, len)
                 });
 
