@@ -4,17 +4,19 @@ pub mod component;
 pub(crate) mod core;
 pub mod dp;
 pub mod memory;
+pub mod sequences;
 pub mod swo;
+mod traits;
 
 pub use communication_interface::{
-    ApInformation, ArmChipInfo, ArmCommunicationInterface, DapAccess, DapError, MemoryApInformation,
+    ApInformation, ArmChipInfo, ArmCommunicationInterface, DapError, MemoryApInformation, Register,
 };
-pub use communication_interface::{PortType, Register};
 pub use swo::{SwoAccess, SwoConfig, SwoMode};
+pub use traits::*;
 
-pub use self::core::m0;
-pub use self::core::m33;
-pub use self::core::m4;
-pub use self::core::CortexDump;
+pub use self::core::armv6m;
+pub use self::core::armv7m;
+pub use self::core::armv8m;
+pub use self::core::Dump;
 
 pub use communication_interface::ArmProbeInterface;
