@@ -8,8 +8,11 @@ use super::{
     dtm::{DmiOperation, DmiOperationStatus, Dtm},
     register, Dmcontrol, Dmstatus,
 };
-use crate::architecture::riscv::*;
 use crate::DebugProbeError;
+use crate::{
+    architecture::riscv::*,
+    probe::{CommandResult, DeferredResultIndex},
+};
 use crate::{MemoryInterface, Probe};
 
 use crate::{probe::JTAGAccess, CoreRegisterAddress, Error as ProbeRsError};
@@ -575,7 +578,8 @@ impl<'probe> RiscvCommunicationInterface {
 
     /// Perform multiple reads from consecutive memory locations
     /// using system bus access.
-    fn perform_memory_read_multiple_sysbus<V: RiscvValue>(
+    /// Only reads up to a width of 32 bits are currently supported.
+    fn perform_memory_read_multiple_sysbus<V: RiscvValue32>(
         &mut self,
         address: u32,
         data: &mut [V],
@@ -589,24 +593,43 @@ impl<'probe> RiscvCommunicationInterface {
         sbcs.set_sbreadondata(true);
         sbcs.set_sbautoincrement(true);
 
-        self.write_dm_register(sbcs)?;
+        self.schedule_write_dm_register(sbcs)?;
 
-        self.write_dm_register(Sbaddress0(address))?;
+        self.schedule_write_dm_register(Sbaddress0(address))?;
 
         let data_len = data.len();
 
-        for value in data[..data_len - 1].iter_mut() {
-            *value = self.read_large_dtm_register::<V, Sbdata>()?;
+        let mut read_results: Vec<usize> = vec![];
+        for _ in data[..data_len - 1].iter() {
+            let idx = self.schedule_read_large_dtm_register::<V, Sbdata>()?;
+            read_results.push(idx);
         }
 
         sbcs.set_sbautoincrement(false);
-        self.write_dm_register(sbcs)?;
+        self.schedule_write_dm_register(sbcs)?;
 
         // Read last value
-        data[data.len() - 1] = self.read_large_dtm_register::<V, Sbdata>()?;
+        read_results.push(self.schedule_read_large_dtm_register::<V, Sbdata>()?);
+
+        let sbcs_result = self.schedule_read_dm_register::<Sbcs>()?;
+
+        let result = self.execute();
+
+        let result = result?;
+        for (out_index, &idx) in read_results.iter().enumerate() {
+            data[out_index] = match result[idx] {
+                CommandResult::U32(data) => V::from_register_value(data),
+                _ => panic!("Internal error occured."),
+            };
+        }
 
         // Check that the read was succesful
-        let sbcs = self.read_dm_register::<Sbcs>()?;
+        let sbcs = match result[sbcs_result] {
+            CommandResult::U32(res) => res,
+            _ => panic!("Internal error occured."),
+        };
+
+        let sbcs = Sbcs(sbcs);
 
         if sbcs.sberror() != 0 {
             Err(RiscvError::SystemBusAccess)
@@ -751,16 +774,26 @@ impl<'probe> RiscvCommunicationInterface {
         sbcs.set_sbaccess(V::WIDTH as u32);
         sbcs.set_sbautoincrement(true);
 
-        self.write_dm_register(sbcs)?;
+        self.schedule_write_dm_register(sbcs)?;
 
-        self.write_dm_register(Sbaddress0(address))?;
+        self.schedule_write_dm_register(Sbaddress0(address))?;
 
         for value in data {
-            self.write_large_dtm_register::<V, Sbdata>(*value)?;
+            self.schedule_write_large_dtm_register::<V, Sbdata>(*value)?;
         }
 
         // Check that the write was succesful
-        let sbcs = self.read_dm_register::<Sbcs>()?;
+        let ok_index = self.schedule_read_dm_register::<Sbcs>()?;
+
+        let result = self.execute()?;
+
+        // Check that the write was succesful
+        let sbcs = match result[ok_index] {
+            CommandResult::U32(res) => res,
+            _ => panic!("Internal error occured."),
+        };
+
+        let sbcs = Sbcs(sbcs);
 
         if sbcs.sberror() != 0 {
             Err(RiscvError::SystemBusAccess)
@@ -921,7 +954,7 @@ impl<'probe> RiscvCommunicationInterface {
         log::debug!("abstractcs: {:?}", abstractcs_prev);
 
         if abstractcs_prev.cmderr() != 0 {
-            //clear previous command error
+            // Clear previous command error.
             let mut abstractcs_clear = Abstractcs(0);
             abstractcs_clear.set_cmderr(0x7);
 
@@ -1204,6 +1237,84 @@ impl<'probe> RiscvCommunicationInterface {
     pub fn close(self) -> Probe {
         Probe::from_attached_probe(self.dtm.probe.into_probe())
     }
+
+    pub(super) fn execute(&mut self) -> Result<Vec<CommandResult>, DebugProbeError> {
+        self.dtm.execute()
+    }
+
+    pub(super) fn schedule_write_dm_register<R: DebugRegister>(
+        &mut self,
+        register: R,
+    ) -> Result<(), DebugProbeError> {
+        // write write command to dmi register
+
+        log::debug!(
+            "Write DM register '{}' at {:#010x} = {:x?}",
+            R::NAME,
+            R::ADDRESS,
+            register
+        );
+
+        self.schedule_write_dm_register_untyped(R::ADDRESS as u64, register.into())?;
+        Ok(())
+    }
+
+    /// Write to a DM register
+    ///
+    /// Use the [`schedule_write_dm_register`] function if possible.
+    fn schedule_write_dm_register_untyped(
+        &mut self,
+        address: u64,
+        value: u32,
+    ) -> Result<DeferredResultIndex, DebugProbeError> {
+        self.dtm
+            .schedule_dmi_register_access(address, value, DmiOperation::Write)
+    }
+
+    pub(super) fn schedule_read_dm_register<R: DebugRegister>(
+        &mut self,
+    ) -> Result<DeferredResultIndex, DebugProbeError> {
+        log::debug!("Reading DM register '{}' at {:#010x}", R::NAME, R::ADDRESS);
+
+        self.schedule_read_dm_register_untyped(R::ADDRESS as u64)
+    }
+
+    /// Read from a DM register
+    ///
+    /// Use the [`schedule_read_dm_register`] function if possible.
+    fn schedule_read_dm_register_untyped(
+        &mut self,
+        address: u64,
+    ) -> Result<DeferredResultIndex, DebugProbeError> {
+        // Prepare the read by sending a read request with the register address
+        self.dtm
+            .schedule_dmi_register_access(address, 0, DmiOperation::Read)?;
+
+        // Read back the response from the previous request.
+        self.dtm
+            .schedule_dmi_register_access(0, 0, DmiOperation::NoOp)
+    }
+
+    fn schedule_read_large_dtm_register<V, R>(
+        &mut self,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        V: RiscvValue,
+        R: LargeRegister,
+    {
+        V::schedule_read_from_register::<R>(self)
+    }
+
+    fn schedule_write_large_dtm_register<V, R>(
+        &mut self,
+        value: V,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        V: RiscvValue,
+        R: LargeRegister,
+    {
+        V::schedule_write_to_register::<R>(self, value)
+    }
 }
 pub(crate) trait LargeRegister {
     const R0_ADDRESS: u8;
@@ -1268,6 +1379,19 @@ pub(crate) trait RiscvValue: std::fmt::Debug + Copy + Sized {
     ) -> Result<(), RiscvError>
     where
         R: LargeRegister;
+
+    fn schedule_read_from_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister;
+
+    fn schedule_write_to_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+        value: Self,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister;
 }
 
 impl RiscvValue for u8 {
@@ -1293,6 +1417,25 @@ impl RiscvValue for u8 {
     {
         interface.write_dm_register_untyped(R::R0_ADDRESS as u64, value as u32)
     }
+
+    fn schedule_read_from_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        interface.schedule_read_dm_register_untyped(R::R0_ADDRESS as u64)
+    }
+
+    fn schedule_write_to_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+        value: Self,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        interface.schedule_write_dm_register_untyped(R::R0_ADDRESS as u64, value as u32)
+    }
 }
 
 impl RiscvValue for u16 {
@@ -1317,6 +1460,25 @@ impl RiscvValue for u16 {
     {
         interface.write_dm_register_untyped(R::R0_ADDRESS as u64, value as u32)
     }
+
+    fn schedule_read_from_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        interface.schedule_read_dm_register_untyped(R::R0_ADDRESS as u64)
+    }
+
+    fn schedule_write_to_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+        value: Self,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        interface.schedule_write_dm_register_untyped(R::R0_ADDRESS as u64, value as u32)
+    }
 }
 
 impl RiscvValue for u32 {
@@ -1339,6 +1501,24 @@ impl RiscvValue for u32 {
     {
         interface.write_dm_register_untyped(R::R0_ADDRESS as u64, value)
     }
+
+    fn schedule_read_from_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        interface.schedule_read_dm_register_untyped(R::R0_ADDRESS as u64)
+    }
+    fn schedule_write_to_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+        value: Self,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        interface.schedule_write_dm_register_untyped(R::R0_ADDRESS as u64, value as u32)
+    }
 }
 
 impl RiscvValue for u64 {
@@ -1374,6 +1554,33 @@ impl RiscvValue for u64 {
         interface.write_dm_register_untyped(R::R1_ADDRESS as u64, upper_bits)?;
         interface.write_dm_register_untyped(R::R0_ADDRESS as u64, lower_bits)
     }
+
+    fn schedule_read_from_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        interface.schedule_read_dm_register_untyped(R::R1_ADDRESS as u64)?;
+        interface.schedule_read_dm_register_untyped(R::R0_ADDRESS as u64)
+    }
+
+    fn schedule_write_to_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+        value: Self,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        let upper_bits = (value >> 32) as u32;
+        let lower_bits = (value & 0xffff_ffff) as u32;
+
+        // R0 has to be written last, side effects are triggerd by writes from
+        // this register.
+
+        interface.schedule_write_dm_register_untyped(R::R1_ADDRESS as u64, upper_bits)?;
+        interface.schedule_write_dm_register_untyped(R::R0_ADDRESS as u64, lower_bits)
+    }
 }
 
 impl RiscvValue for u128 {
@@ -1418,6 +1625,39 @@ impl RiscvValue for u128 {
         interface.write_dm_register_untyped(R::R1_ADDRESS as u64, bits_1)?;
         interface.write_dm_register_untyped(R::R0_ADDRESS as u64, bits_0)
     }
+
+    fn schedule_read_from_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        interface.schedule_read_dm_register_untyped(R::R3_ADDRESS as u64)?;
+        interface.schedule_read_dm_register_untyped(R::R2_ADDRESS as u64)?;
+        interface.schedule_read_dm_register_untyped(R::R1_ADDRESS as u64)?;
+        interface.schedule_read_dm_register_untyped(R::R0_ADDRESS as u64)
+    }
+
+    fn schedule_write_to_register<R>(
+        interface: &mut RiscvCommunicationInterface,
+        value: Self,
+    ) -> Result<DeferredResultIndex, DebugProbeError>
+    where
+        R: LargeRegister,
+    {
+        let bits_3 = (value >> 96) as u32;
+        let bits_2 = (value >> 64) as u32;
+        let bits_1 = (value >> 32) as u32;
+        let bits_0 = (value & 0xffff_ffff) as u32;
+
+        // R0 has to be written last, side effects are triggerd by writes from
+        // this register.
+
+        interface.schedule_write_dm_register_untyped(R::R3_ADDRESS as u64, bits_3)?;
+        interface.schedule_write_dm_register_untyped(R::R2_ADDRESS as u64, bits_2)?;
+        interface.schedule_write_dm_register_untyped(R::R1_ADDRESS as u64, bits_1)?;
+        interface.schedule_write_dm_register_untyped(R::R0_ADDRESS as u64, bits_0)
+    }
 }
 
 impl MemoryInterface for RiscvCommunicationInterface {
