@@ -21,6 +21,7 @@ pub(crate) mod assembly;
 mod dtm;
 
 pub mod communication_interface;
+pub mod sequences;
 
 pub struct Riscv32<'probe> {
     interface: &'probe mut RiscvCommunicationInterface,
@@ -491,10 +492,10 @@ impl<'probe> CoreInterface for Riscv32<'probe> {
         let mut breakpoints = vec![];
         let num_hw_breakpoints = self.get_available_breakpoint_units()? as usize;
         for bp_unit_index in 0..num_hw_breakpoints {
-            //select the trigger
+            // Select the trigger.
             self.write_csr(tselect, bp_unit_index as u32)?;
 
-            //Read the trigger "configuration" data
+            // Read the trigger "configuration" data.
             let tdata_value = Mcontrol(self.read_csr(tdata1)?);
 
             log::warn!("Breakpoint {}: {:?}", bp_unit_index, tdata_value);
@@ -505,7 +506,7 @@ impl<'probe> CoreInterface for Riscv32<'probe> {
             let trigger_any_action_enabled =
                 tdata_value.execute() || tdata_value.store() || tdata_value.load();
 
-            //Only return if the trigger if it is for an execution debug action in all modes
+            // Only return if the trigger if it is for an execution debug action in all modes.
             if tdata_value.type_() == 0b10
                 && tdata_value.action() == 1
                 && tdata_value.match_() == 0
