@@ -1,12 +1,15 @@
 pub(crate) mod communication_interface;
 
 pub use communication_interface::CommunicationInterface;
+pub use probe_rs_target::Architecture;
+use probe_rs_target::CoreType;
 
 use crate::architecture::{
-    arm::core::CortexState, riscv::communication_interface::RiscvCommunicationInterface,
+    arm::core::State, riscv::communication_interface::RiscvCommunicationInterface,
 };
-use crate::config::CoreType;
-use crate::{error, Error, Memory, MemoryInterface};
+use crate::error;
+use crate::Target;
+use crate::{Error, Memory, MemoryInterface};
 use anyhow::{anyhow, Result};
 use std::time::Duration;
 
@@ -66,11 +69,11 @@ pub(crate) enum RegisterKind {
 }
 
 /// Register description for a core.
-
 #[derive(Debug)]
 pub struct RegisterFile {
     pub(crate) platform_registers: &'static [RegisterDescription],
 
+    /// Register description for the program counter
     pub(crate) program_counter: &'static RegisterDescription,
 
     pub(crate) stack_pointer: &'static RegisterDescription,
@@ -87,15 +90,15 @@ impl RegisterFile {
     }
 
     pub fn program_counter(&self) -> &RegisterDescription {
-        &self.program_counter
+        self.program_counter
     }
 
     pub fn stack_pointer(&self) -> &RegisterDescription {
-        &self.stack_pointer
+        self.stack_pointer
     }
 
     pub fn return_address(&self) -> &RegisterDescription {
-        &self.return_address
+        self.return_address
     }
 
     pub fn argument_register(&self, index: usize) -> &RegisterDescription {
@@ -125,17 +128,17 @@ impl RegisterFile {
 
 pub trait CoreInterface: MemoryInterface {
     /// Wait until the core is halted. If the core does not halt on its own,
-    /// a [DebugProbeError::Timeout] error will be returned.
+    /// a [`DebugProbeError::Timeout`](crate::DebugProbeError::Timeout) error will be returned.
     fn wait_for_core_halted(&mut self, timeout: Duration) -> Result<(), error::Error>;
 
     /// Check if the core is halted. If the core does not halt on its own,
-    /// a [DebugProbeError::Timeout] error will be returned.
+    /// a [`DebugProbeError::Timeout`](crate::DebugProbeError::Timeout) error will be returned.
     fn core_halted(&mut self) -> Result<bool, error::Error>;
 
     fn status(&mut self) -> Result<CoreStatus, error::Error>;
 
     /// Try to halt the core. This function ensures the core is actually halted, and
-    /// returns a [DebugProbeError::Timeout] otherwise.
+    /// returns a [`DebugProbeError::Timeout`](crate::DebugProbeError::Timeout) otherwise.
     fn halt(&mut self, timeout: Duration) -> Result<CoreInformation, error::Error>;
 
     fn run(&mut self) -> Result<(), error::Error>;
@@ -227,59 +230,70 @@ impl CoreState {
     pub fn new(id: usize) -> Self {
         Self { id }
     }
+
+    pub fn id(&self) -> usize {
+        self.id
+    }
 }
 
 #[derive(Debug)]
 pub enum SpecificCoreState {
-    M3(CortexState),
-    M4(CortexState),
-    M33(CortexState),
-    M0(CortexState),
-    M7(CortexState),
+    Armv6m(State),
+    Armv7m(State),
+    Armv7em(State),
+    Armv8m(State),
     Riscv,
 }
 
 impl SpecificCoreState {
     pub(crate) fn from_core_type(typ: CoreType) -> Self {
         match typ {
-            CoreType::M0 => SpecificCoreState::M0(CortexState::new()),
-            CoreType::M3 => SpecificCoreState::M3(CortexState::new()),
-            CoreType::M33 => SpecificCoreState::M33(CortexState::new()),
-            CoreType::M4 => SpecificCoreState::M4(CortexState::new()),
-            CoreType::M7 => SpecificCoreState::M7(CortexState::new()),
+            CoreType::Armv6m => SpecificCoreState::Armv6m(State::new()),
+            CoreType::Armv7m => SpecificCoreState::Armv7m(State::new()),
+            CoreType::Armv7em => SpecificCoreState::Armv7m(State::new()),
+            CoreType::Armv8m => SpecificCoreState::Armv8m(State::new()),
             CoreType::Riscv => SpecificCoreState::Riscv,
         }
     }
 
     pub(crate) fn core_type(&self) -> CoreType {
         match self {
-            SpecificCoreState::M0(_) => CoreType::M0,
-            SpecificCoreState::M3(_) => CoreType::M3,
-            SpecificCoreState::M33(_) => CoreType::M33,
-            SpecificCoreState::M4(_) => CoreType::M4,
-            SpecificCoreState::M7(_) => CoreType::M7,
+            SpecificCoreState::Armv6m(_) => CoreType::Armv6m,
+            SpecificCoreState::Armv7m(_) => CoreType::Armv7m,
+            SpecificCoreState::Armv7em(_) => CoreType::Armv7em,
+            SpecificCoreState::Armv8m(_) => CoreType::Armv8m,
             SpecificCoreState::Riscv => CoreType::Riscv,
         }
     }
 
-    pub(crate) fn attach_arm<'probe>(
+    pub(crate) fn attach_arm<'probe, 'target: 'probe>(
         &'probe mut self,
         state: &'probe mut CoreState,
         memory: Memory<'probe>,
+        target: &'target Target,
     ) -> Result<Core<'probe>, Error> {
+        let debug_sequence = match &target.debug_sequence {
+            crate::config::DebugSequence::Arm(sequence) => sequence.clone(),
+            crate::config::DebugSequence::Riscv(_) => {
+                return Err(Error::UnableToOpenProbe(
+                    "Core architecture and Probe mismatch.",
+                ))
+            }
+        };
+
         Ok(match self {
-            // TODO: Change this once the new archtecture structure for ARM hits.
-            // Cortex-M3, M4 and M7 use the Armv7[E]-M architecture and are
-            // identical for our purposes.
-            SpecificCoreState::M3(s) | SpecificCoreState::M4(s) | SpecificCoreState::M7(s) => {
-                Core::new(crate::architecture::arm::m4::M4::new(memory, s)?, state)
-            }
-            SpecificCoreState::M33(s) => {
-                Core::new(crate::architecture::arm::m33::M33::new(memory, s)?, state)
-            }
-            SpecificCoreState::M0(s) => {
-                Core::new(crate::architecture::arm::m0::M0::new(memory, s)?, state)
-            }
+            SpecificCoreState::Armv6m(s) => Core::new(
+                crate::architecture::arm::armv6m::Armv6m::new(memory, s, debug_sequence)?,
+                state,
+            ),
+            SpecificCoreState::Armv7m(s) | SpecificCoreState::Armv7em(s) => Core::new(
+                crate::architecture::arm::armv7m::Armv7m::new(memory, s, debug_sequence)?,
+                state,
+            ),
+            SpecificCoreState::Armv8m(s) => Core::new(
+                crate::architecture::arm::armv8m::Armv8m::new(memory, s, debug_sequence)?,
+                state,
+            ),
             _ => {
                 return Err(Error::UnableToOpenProbe(
                     "Core architecture and Probe mismatch.",
@@ -328,19 +342,19 @@ impl<'probe> Core<'probe> {
     }
 
     /// Wait until the core is halted. If the core does not halt on its own,
-    /// a [DebugProbeError::Timeout] error will be returned.
+    /// a [`DebugProbeError::Timeout`](crate::DebugProbeError::Timeout) error will be returned.
     pub fn wait_for_core_halted(&mut self, timeout: Duration) -> Result<(), error::Error> {
         self.inner.wait_for_core_halted(timeout)
     }
 
     /// Check if the core is halted. If the core does not halt on its own,
-    /// a [DebugProbeError::Timeout] error will be returned.
+    /// a [`DebugProbeError::Timeout`](crate::DebugProbeError::Timeout) error will be returned.
     pub fn core_halted(&mut self) -> Result<bool, error::Error> {
         self.inner.core_halted()
     }
 
     /// Try to halt the core. This function ensures the core is actually halted, and
-    /// returns a [DebugProbeError::Timeout] otherwise.
+    /// returns a [`DebugProbeError::Timeout`](crate::DebugProbeError::Timeout) otherwise.
     pub fn halt(&mut self, timeout: Duration) -> Result<CoreInformation, error::Error> {
         self.inner.halt(timeout)
     }
@@ -426,7 +440,7 @@ impl<'probe> Core<'probe> {
             self.enable_breakpoints(true)?;
         }
 
-        //If there is a breakpoint set already, return its bp_unit_index, else find the next free index
+        // If there is a breakpoint set already, return its bp_unit_index, else find the next free index.
         let breakpoint_comparator_index = match self
             .inner
             .get_hw_breakpoints()?
@@ -478,7 +492,7 @@ impl<'probe> Core<'probe> {
     ///
     /// This function will clear all HW breakpoints which are configured on the target,
     /// regardless if they are set by probe-rs, AND regardless if they are enabled or not.
-    /// Also used as a helper function in [`Session::drop`].
+    /// Also used as a helper function in [`Session::drop`](crate::session::Session).
     pub fn clear_all_hw_breakpoints(&mut self) -> Result<(), error::Error> {
         for breakpoint in (self.inner.get_hw_breakpoints()?).into_iter().flatten() {
             self.clear_hw_breakpoint(breakpoint)?
@@ -502,7 +516,7 @@ impl<'probe> CoreList<'probe> {
 impl<'probe> std::ops::Deref for CoreList<'probe> {
     type Target = [CoreType];
     fn deref(&self) -> &Self::Target {
-        &self.0
+        self.0
     }
 }
 
@@ -521,12 +535,6 @@ pub struct Breakpoint {
     register_hw: usize,
 }
 
-#[derive(Clone, Copy, Debug, PartialEq)]
-pub enum Architecture {
-    Arm,
-    Riscv,
-}
-
 #[derive(Debug, PartialEq, Copy, Clone)]
 pub enum CoreStatus {
     Running,
