@@ -83,44 +83,48 @@ impl Variable {
             name: String::new(),
             value: String::new(),
             file: String::new(),
-            /// There are instances when extract_location() will encounter a value in the DWARF definition, rather than a memory location where the value can be read. In those cases it will set Variable.value, and set Variable.location to u64::MAX, which tells the Variable.extract_value() to NOT overwrite it
+            /// There are instances when extract_location() will encounter a value in the DWARF definition, rather than a memory location where the value can be read.
+            /// In those cases it will set Variable.value, and set Variable.location to u64::MAX, which tells the Variable.extract_value() to NOT overwrite it.
             memory_location: 0,
             ..Default::default()
         }
     }
 
-    /// Implementing set_value(), because the library passes errors into the value of the variable. This ensures debug front ends can see the errors, but doesn't fail because of a single variable not being able to decode correctly.
+    /// Implementing set_value(), because the library passes errors into the value of the variable.
+    /// This ensures debug front ends can see the errors, but doesn't fail because of a single variable not being able to decode correctly.
     pub fn set_value(&mut self, new_value: String) {
         if self.value.is_empty() {
             self.value = new_value;
         } else {
-            //We append the new value to the old value, so that we don't loose any prior errors or warnings originating from the process of decoding the actual value
+            // We append the new value to the old value, so that we don't loose any prior errors or warnings originating from the process of decoding the actual value.
             self.value = format!("{} : {}", self.value, new_value);
         }
     }
 
-    /// Implementing get_value(), because Variable.value has to be private (a requirement of updating the value without overriding earlier values ... see set_value())
+    /// Implementing get_value(), because Variable.value has to be private (a requirement of updating the value without overriding earlier values ... see set_value()).
     pub fn get_value(&self) -> String {
         self.value.clone()
     }
 
     /// Evaluate the variable's result if possible and set self.value, or else set self.value as the error String.
     pub fn extract_value(&mut self, core: &mut Core<'_>) {
-        //Since extract_value is called very late in the decoding process, we can defer setting of the VariableKind until this point
+        // Since extract_value is called very late in the decoding process, we can defer setting of the VariableKind until this point.
         if self.name.starts_with("__") {
             self.kind = VariableKind::Indexed
         } else {
             self.kind = VariableKind::Named
         }
-        //Quick exit if we don't really need to do much more
-        if self.memory_location == u64::MAX// the value was set by get_location(), so just leave it as is
-        || !self.value.is_empty()// the value was set elsewhere in this library - probably because of an error - so just leave it as is
-        || self.memory_location.is_zero()
+        // Quick exit if we don't really need to do much more.
+        // The value was set by get_location(), so just leave it as is.
+        if self.memory_location == u64::MAX
+        // The value was set elsewhere in this library - probably because of an error - so just leave it as is.
+        || !self.value.is_empty()
         // Templates, Phantoms, etc.
+        || self.memory_location.is_zero()
         {
             return;
         }
-        //This is the primary logic for decoding a variable's value, once we know the type and memory_location
+        // This is the primary logic for decoding a variable's value, once we know the type and memory_location.
         let string_value = match self.type_name.as_str() {
             "!" => "<Never returns>".to_string(),
             "()" => "()".to_string(),
@@ -131,7 +135,8 @@ impl Variable {
             "&str" => {
                 let string_value = String::get_value(self, core)
                     .map_or_else(|err| format!("ERROR: {:?}", err), |value| value);
-                self.children = None; //We don't need these for debugging purposes ... unless we get the ERROR below
+                // We don't need these for debugging purposes ... unless we get the ERROR below.
+                self.children = None;
                 string_value
             }
             "i8" => i8::get_value(self, core)
@@ -187,36 +192,33 @@ impl Variable {
                 |value| value.separate_with_underscores(),
             ),
             "None" => "None".to_string(),
-            oops => {
-                match &self.children {
-                    Some(_children) => {
-                        if oops.is_empty() {
-                            "ERROR: This is a bug! Attempted to evaluate an empty Type".to_string()
-                        } else {
-                            format!("{}", self) //Use the Display implementation below to create 'at a glance' values for structured types
-                                                //oops.to_string() //return the type name as the value for non-leaf level variables
-                        }
-                    }
-                    None => {
-                        format!(
-                            "UNIMPLEMENTED: Evaluate type {} of ({} bytes) at location 0x{:08x}",
-                            oops, self.byte_size, self.memory_location
-                        )
+            oops => match &self.children {
+                Some(_children) => {
+                    if oops.is_empty() {
+                        "ERROR: This is a bug! Attempted to evaluate an empty Type".to_string()
+                    } else {
+                        format!("{}", self)
                     }
                 }
-            }
+                None => {
+                    format!(
+                        "UNIMPLEMENTED: Evaluate type {} of ({} bytes) at location 0x{:08x}",
+                        oops, self.byte_size, self.memory_location
+                    )
+                }
+            },
         };
-        // println!("!!!\t{}\t{}\t{}", self.name, self.location, string_value);
         self.value = string_value;
     }
 
     /// Instead of just pushing to Variable.children, do some intelligent selection/addition of new Variables.
-    /// Primarily this is to force late-as-possible(before parent) call of `extract_value()` on child variables, and to determine which of the processed DWARF nodes are included in the final variable tree
+    /// Primarily this is to force late-as-possible(before parent) call of `extract_value()` on child variables,
+    /// and to determine which of the processed DWARF nodes are included in the final variable tree.
     pub fn add_child_variable(&mut self, child_variable: &mut Variable, core: &mut Core<'_>) {
         if !(child_variable.inclusion == VariableInclusion::Undetermined
             || child_variable.inclusion == VariableInclusion::Exclude)
         {
-            //Just-in-Time creation of Vec to store the children
+            // Just-in-Time creation of Vec to store the children.
             let children: &mut Vec<Variable> = match &mut self.children {
                 Some(children) => children,
                 None => {
@@ -224,16 +226,18 @@ impl Variable {
                     self.children.as_mut().unwrap()
                 }
             };
-            child_variable.extract_value(core); //Warning, child_variable's VariableInclusion might have changed after this line
-                                                //Ensure parent inclusion setting honours the child inclusion
+            // Warning, child_variable's VariableInclusion might have changed after this line.
+            // Ensure parent inclusion setting honours the child inclusion.
+            child_variable.extract_value(core);
             self.inclusion = VariableInclusion::Include;
             if child_variable.inclusion == VariableInclusion::Include {
-                //Check to see if this child already exists - We need to do this, because cargo's `codegen-units` sometimes spread and/or repeat namespace children between them
+                // Check to see if this child already exists - We need to do this,
+                // because cargo's `codegen-units` sometimes spread and/or repeat namespace children between them.
                 if let Some(existing_child) = children.iter_mut().find(|current_child| {
                     current_child.name == child_variable.name
                         && current_child.type_name == child_variable.type_name
                 }) {
-                    //Just add the children (if there are any) from the new child to the existing child
+                    // Just add the children (if there are any) from the new child to the existing child
                     if let Some(new_children) = child_variable.children.clone() {
                         for mut new_child in new_children {
                             existing_child.add_child_variable(&mut new_child, core);
@@ -250,29 +254,29 @@ impl Variable {
 impl fmt::Display for Variable {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         if self.value.is_empty() {
-            //Only do this if we do not already have a value assigned
+            // Only do this if we do not already have a value assigned.
             if let Some(children) = self.children.clone() {
-                //Make sure we can safely unwrap() children
+                // Make sure we can safely unwrap() children.
                 if self.type_name.starts_with('&') {
-                    //Pointers
+                    // Pointers
                     write!(f, "{}", children.first().unwrap())
                 } else if self.type_name.starts_with('(') {
-                    //Tuples
+                    // Tuples
                     write!(f, "(")?;
                     for child in children {
                         write!(f, "{}, ", child)?;
                     }
                     write!(f, ")")
                 } else if self.type_name.starts_with('[') {
-                    //Arrays
+                    // Arrays
                     write!(f, "[")?;
                     for child in children {
                         write!(f, "{}, ", child)?;
                     }
                     write!(f, "]")
                 } else {
-                    //Generic handling of other structured types
-                    //TODO: This is 'ok' for most, but could benefit from some custom formatting, e.g. Unions
+                    // Generic handling of other structured types.
+                    // TODO: This is 'ok' for most, but could benefit from some custom formatting, e.g. Unions.
                     if self.kind == VariableKind::Named {
                         write!(f, "{}:{{", self.name)?;
                     } else {
@@ -284,14 +288,17 @@ impl fmt::Display for Variable {
                     write!(f, "}}")
                 }
             } else {
-                write!(f, "{}", self.type_name) //Unknown
+                // Unknown.
+                write!(f, "{}", self.type_name)
             }
         } else {
-            write!(f, "{}", self.value) //Use the supplied value
+            // Use the supplied value.
+            write!(f, "{}", self.value)
         }
     }
 }
-/// Traits and Impl's to read from memory and decode the Variable value based on Variable::typ and Variable::location. The MS DAP protocol passes the value as a string, so these are here only to provide the memory read logic before returning it as a string.
+/// Traits and Impl's to read from memory and decode the Variable value based on Variable::typ and Variable::location.
+/// The MS DAP protocol passes the value as a string, so these are here only to provide the memory read logic before returning it as a string.
 trait Value {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError>
     where
@@ -308,7 +315,8 @@ impl Value for bool {
 impl Value for char {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mem_data = core.read_word_32(variable.memory_location as u32)?;
-        let ret_value: char = mem_data.try_into()?; //TODO: Use char::from_u32 once it stabilizes
+        // TODO: Use char::from_u32 once it stabilizes.
+        let ret_value: char = mem_data.try_into()?;
         Ok(ret_value)
     }
 }
@@ -347,7 +355,7 @@ impl Value for String {
                     str_value = "ERROR: Failed to determine &str memory location".to_string();
                 } else {
                     let mut buff = vec![0u8; string_length];
-                    core.read_8(string_location as u32, &mut buff)?;
+                    core.read(string_location as u32, &mut buff)?;
                     str_value = core::str::from_utf8(&buff)?.to_owned();
                 }
             }
@@ -361,7 +369,7 @@ impl Value for String {
 impl Value for i8 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 1];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = i8::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -369,7 +377,7 @@ impl Value for i8 {
 impl Value for i16 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 2];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = i16::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -377,7 +385,7 @@ impl Value for i16 {
 impl Value for i32 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 4];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = i32::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -385,7 +393,7 @@ impl Value for i32 {
 impl Value for i64 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 8];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = i64::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -393,7 +401,7 @@ impl Value for i64 {
 impl Value for i128 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 16];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = i128::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -401,8 +409,9 @@ impl Value for i128 {
 impl Value for isize {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 4];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
-        let ret_value = i32::from_le_bytes(buff); //TODO: how to get the MCU isize calculated for all platforms
+        core.read(variable.memory_location as u32, &mut buff)?;
+        // TODO: how to get the MCU isize calculated for all platforms.
+        let ret_value = i32::from_le_bytes(buff);
         Ok(ret_value as isize)
     }
 }
@@ -410,7 +419,7 @@ impl Value for isize {
 impl Value for u8 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 1];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = u8::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -418,7 +427,7 @@ impl Value for u8 {
 impl Value for u16 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 2];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = u16::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -426,7 +435,7 @@ impl Value for u16 {
 impl Value for u32 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 4];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = u32::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -434,7 +443,7 @@ impl Value for u32 {
 impl Value for u64 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 8];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = u64::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -442,7 +451,7 @@ impl Value for u64 {
 impl Value for u128 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 16];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = u128::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -450,15 +459,16 @@ impl Value for u128 {
 impl Value for usize {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 4];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
-        let ret_value = u32::from_le_bytes(buff); //TODO: how to get the MCU usize calculated for all platforms
+        core.read(variable.memory_location as u32, &mut buff)?;
+        // TODO: how to get the MCU usize calculated for all platforms.
+        let ret_value = u32::from_le_bytes(buff);
         Ok(ret_value as usize)
     }
 }
 impl Value for f32 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 4];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = f32::from_le_bytes(buff);
         Ok(ret_value)
     }
@@ -466,7 +476,7 @@ impl Value for f32 {
 impl Value for f64 {
     fn get_value(variable: &Variable, core: &mut Core<'_>) -> Result<Self, DebugError> {
         let mut buff = [0u8; 8];
-        core.read_8(variable.memory_location as u32, &mut buff)?;
+        core.read(variable.memory_location as u32, &mut buff)?;
         let ret_value = f64::from_le_bytes(buff);
         Ok(ret_value)
     }
