@@ -53,31 +53,42 @@ impl Operation for Verify {
 /// Once constructed it can be used to program date to the flash.
 pub(super) struct Flasher<'session> {
     session: &'session mut Session,
+    core_index: usize,
     flash_algorithm: FlashAlgorithm,
 }
 
 impl<'session> Flasher<'session> {
     pub(super) fn new(
         session: &'session mut Session,
+        core_index: usize,
         raw_flash_algorithm: &RawFlashAlgorithm,
     ) -> Result<Self, FlashError> {
         let target = session.target();
 
+        // Find a RAM region from which we can run the algo.
         let mm = &target.memory_map;
+        let core_name = &target.cores[core_index].name;
         let ram = mm
             .iter()
-            .find_map(|mm| match mm {
+            .filter_map(|mm| match mm {
                 MemoryRegion::Ram(ram) => Some(ram),
                 _ => None,
             })
+            .find(|ram| {
+                // The RAM must be accessible from the core we're going to run the algo on.
+                ram.cores.contains(core_name)
+            })
             .ok_or(FlashError::NoRamDefined {
-                chip: target.name.clone(),
+                chip: session.target().name.clone(),
             })?;
 
+        log::info!("chosen RAM to run the algo: {:x?}", ram);
+
         let flash_algorithm = FlashAlgorithm::assemble_from_raw(raw_flash_algorithm, ram, target)?;
 
         let mut this = Self {
             session,
+            core_index,
             flash_algorithm,
         };
 
@@ -99,10 +110,13 @@ impl<'session> Flasher<'session> {
         let algo = &mut self.flash_algorithm;
 
         // Attach to memory and core.
-        let mut core = self.session.core(0).map_err(FlashError::Core)?;
+        let mut core = self
+            .session
+            .core(self.core_index)
+            .map_err(FlashError::Core)?;
 
         // TODO: Halt & reset target.
-        log::debug!("Halting core.");
+        log::debug!("Halting core {}", self.core_index);
         let cpu_info = core
             .halt(Duration::from_millis(100))
             .map_err(FlashError::Core)?;
@@ -153,7 +167,10 @@ impl<'session> Flasher<'session> {
         clock: Option<u32>,
     ) -> Result<ActiveFlasher<'_, O>, FlashError> {
         // Attach to memory and core.
-        let core = self.session.core(0).map_err(FlashError::Core)?;
+        let core = self
+            .session
+            .core(self.core_index)
+            .map_err(FlashError::Core)?;
 
         log::debug!("Preparing Flasher for operation {}", O::operation_name());
         let mut flasher = ActiveFlasher::<O> {
@@ -288,7 +305,7 @@ impl<'session> Flasher<'session> {
         self.run_verify(|active| {
             active
                 .core
-                .read_8(fill.address(), page_slice)
+                .read(fill.address(), page_slice)
                 .map_err(FlashError::Core)
         })
     }
@@ -788,12 +805,23 @@ impl<'p> ActiveFlasher<'p, Program> {
         );
 
         // TODO: Prevent security settings from locking the device.
-
         // Transfer the buffer bytes to RAM.
+        let words: Vec<u32> = bytes
+            .chunks_exact(core::mem::size_of::<u32>())
+            .map(|a| u32::from_le_bytes([a[0], a[1], a[2], a[3]]))
+            .collect();
+
+        let t1 = std::time::Instant::now();
         self.core
-            .write_8(algo.page_buffers[buffer_number], bytes)
+            .write_32(algo.page_buffers[buffer_number], &words)
             .map_err(FlashError::Core)?;
 
+        log::info!(
+            "Took {:?} to download {} byte page into ram",
+            t1.elapsed(),
+            bytes.len()
+        );
+
         Ok(())
     }
 }
