@@ -1,32 +1,30 @@
 pub(crate) mod cmsisdap;
+pub(crate) mod espusbjtag;
+pub(crate) mod fake_probe;
 #[cfg(feature = "ftdi")]
 pub(crate) mod ftdi;
 pub(crate) mod jlink;
 pub(crate) mod stlink;
 
-use crate::architecture::arm::ap::RawApAccess;
-use crate::{architecture::arm::ap::AccessPort, Session};
-use crate::{
-    architecture::arm::memory::adi_v5_memory_interface::ADIMemoryInterface,
-    config::{RegistryError, TargetSelector},
-};
-use crate::{
-    architecture::arm::{ap::MemoryAp, MemoryApInformation},
-    error::Error,
-};
-use crate::{
-    architecture::{
-        arm::{
-            ap::memory_ap::mock::MockMemoryAp, communication_interface::ArmProbeInterface,
-            DapAccess, PortType, SwoAccess,
-        },
-        riscv::communication_interface::RiscvCommunicationInterface,
+use crate::architecture::{
+    arm::{
+        communication_interface::DapProbe,
+        sequences::{ArmDebugSequence, DefaultArmSequence},
+        PortType, SwoAccess,
     },
-    Memory,
+    riscv::communication_interface::RiscvCommunicationInterface,
+};
+use crate::error::Error;
+use crate::Session;
+use crate::{
+    architecture::arm::communication_interface::UninitializedArmProbe,
+    config::{RegistryError, TargetSelector},
 };
 use jlink::list_jlink_devices;
 use std::{convert::TryFrom, fmt};
 
+use self::espusbjtag::list_espjtag_devices;
+
 /// Used to log warnings when the measured target voltage is
 /// lower than 1.4V, if at all measureable.
 const LOW_TARGET_VOLTAGE_WARNING_THRESHOLD: f32 = 1.4;
@@ -117,10 +115,12 @@ pub enum DebugProbeError {
     TargetNotFound,
     #[error("Some functionality was not implemented yet: {0}")]
     NotImplemented(&'static str),
+    #[error("This debug sequence is not supported on the used probe: {0}")]
+    DebugSequenceNotSupported(&'static str),
     #[error("Error in previous batched command")]
     BatchError(BatchCommand),
-    #[error("Command not supported by probe")]
-    CommandNotSupportedByProbe,
+    #[error("Command not supported by probe: {0}")]
+    CommandNotSupportedByProbe(&'static str),
     #[error("Unable to set hardware breakpoint, all available breakpoint units are in use.")]
     BreakpointUnitsExceeded,
     #[error(transparent)]
@@ -200,6 +200,8 @@ impl Probe {
 
         list.extend(list_jlink_devices());
 
+        list.extend(list_espjtag_devices());
+
         list
     }
 
@@ -223,7 +225,12 @@ impl Probe {
             Err(DebugProbeError::ProbeCouldNotBeCreated(ProbeCreationError::NotFound)) => {}
             Err(e) => return Err(e),
         };
-        match jlink::JLink::new_from_selector(selector) {
+        match jlink::JLink::new_from_selector(selector.clone()) {
+            Ok(link) => return Ok(Probe::from_specific_probe(link)),
+            Err(DebugProbeError::ProbeCouldNotBeCreated(ProbeCreationError::NotFound)) => {}
+            Err(e) => return Err(e),
+        };
+        match espusbjtag::EspUsbJtag::new_from_selector(selector) {
             Ok(link) => return Ok(Probe::from_specific_probe(link)),
             Err(DebugProbeError::ProbeCouldNotBeCreated(ProbeCreationError::NotFound)) => {}
             Err(e) => return Err(e),
@@ -234,59 +241,6 @@ impl Probe {
         ))
     }
 
-    // /// Tries to mass erase a locked nRF52 chip, this process may timeout, if it does, the chip
-    // /// might be unlocked or not, it is advised to try again if flashing fails
-    // pub fn nrf_recover(&mut self) -> Result<(), DebugProbeError> {
-    //     let ctrl_port = match get_ap_by_idr(self, |idr| idr == CTRL_AP_IDR) {
-    //         Some(port) => CtrlAP::from(port),
-    //         None => {
-    //             return Err(DebugProbeError::AccessPortError(
-    //                 AccessPortError::CtrlAPNotFound,
-    //             ));
-    //         }
-    //     };
-    //     log::info!("Starting mass erase...");
-    //     let mut erase_reg = ERASEALL::from(1);
-    //     let status_reg = ERASEALLSTATUS::from(0);
-    //     let mut reset_reg = RESET::from(1);
-
-    //     // Reset first
-    //     self.write_ap_register(ctrl_port, reset_reg)?;
-    //     reset_reg.RESET = false;
-    //     self.write_ap_register(ctrl_port, reset_reg)?;
-
-    //     self.write_ap_register(ctrl_port, erase_reg)?;
-
-    //     // Prepare timeout
-    //     let now = Instant::now();
-    //     let status = self.read_ap_register(ctrl_port, status_reg)?;
-    //     log::info!("Erase status: {:?}", status.ERASEALLSTATUS);
-    //     let timeout = loop {
-    //         let status = self.read_ap_register(ctrl_port, status_reg)?;
-    //         if !status.ERASEALLSTATUS {
-    //             break false;
-    //         }
-    //         if now.elapsed().as_secs() >= UNLOCK_TIMEOUT {
-    //             break true;
-    //         }
-    //     };
-    //     reset_reg.RESET = true;
-    //     self.write_ap_register(ctrl_port, reset_reg)?;
-    //     reset_reg.RESET = false;
-    //     self.write_ap_register(ctrl_port, reset_reg)?;
-    //     erase_reg.ERASEALL = false;
-    //     self.write_ap_register(ctrl_port, erase_reg)?;
-    //     if timeout {
-    //         log::error!(
-    //             "    {} Mass erase process timeout, the chip might still be locked.",
-    //             "Error".red().bold()
-    //         );
-    //     } else {
-    //         log::info!("Mass erase completed, chip unlocked");
-    //     }
-    //     Ok(())
-    // }
-
     /// Get human readable name for the probe
     pub fn get_name(&self) -> String {
         self.inner.get_name().to_string()
@@ -298,10 +252,9 @@ impl Probe {
     ///
     /// If this doesn't work, you might want to try `attach_under_reset`
     pub fn attach(mut self, target: impl Into<TargetSelector>) -> Result<Session, Error> {
-        self.inner.attach()?;
         self.attached = true;
 
-        Session::new(self, target, AttachMethod::Normal)
+        Session::new(self, target.into(), AttachMethod::Normal)
     }
 
     pub fn attach_to_unspecified(&mut self) -> Result<(), Error> {
@@ -310,6 +263,22 @@ impl Probe {
         Ok(())
     }
 
+    pub fn attach_to_unspecified_under_reset(&mut self) -> Result<(), Error> {
+        if let Some(dap_probe) = self.try_as_dap_probe() {
+            DefaultArmSequence(()).reset_hardware_assert(dap_probe)?;
+        } else {
+            log::info!(
+                "Custom reset sequences are not supported on {}.",
+                self.get_name()
+            );
+            log::info!("Falling back to standard probe reset.");
+            self.target_reset_assert()?;
+        }
+
+        self.inner_attach()?;
+        Ok(())
+    }
+
     /// Attach to the chip under hard-reset.
     ///
     /// This asserts the reset pin via the probe, plays the protocol init routines and deasserts the pin.
@@ -319,15 +288,14 @@ impl Probe {
         mut self,
         target: impl Into<TargetSelector>,
     ) -> Result<Session, Error> {
-        log::debug!("Asserting reset");
-        self.inner.target_reset_assert()?;
-
-        self.inner.attach()?;
-
         self.attached = true;
 
         // The session will de-assert reset after connecting to the debug interface.
-        Session::new(self, target, AttachMethod::UnderReset)
+        Session::new(self, target.into(), AttachMethod::UnderReset)
+    }
+
+    pub(crate) fn inner_attach(&mut self) -> Result<(), DebugProbeError> {
+        self.inner.attach()
     }
 
     /// Selects the transport protocol to be used by the debug probe.
@@ -351,6 +319,11 @@ impl Probe {
         self.inner.target_reset()
     }
 
+    pub(crate) fn target_reset_assert(&mut self) -> Result<(), DebugProbeError> {
+        log::debug!("Asserting target reset");
+        self.inner.target_reset_assert()
+    }
+
     pub fn target_reset_deassert(&mut self) -> Result<(), DebugProbeError> {
         log::debug!("Deasserting target reset");
         self.inner.target_reset_deassert()
@@ -376,13 +349,13 @@ impl Probe {
         self.inner.has_arm_interface()
     }
 
-    /// Try to get a trait object implementing [`ArmProbeInterface`], which can
+    /// Try to get a trait object implementing `UninitializedArmProbe`, which can
     /// can be used to communicate with chips using the ARM architecture.
     ///
     /// If an error occurs while trying to connect, the probe is returned.
     pub fn try_into_arm_interface<'probe>(
         self,
-    ) -> Result<Box<dyn ArmProbeInterface + 'probe>, (Self, DebugProbeError)> {
+    ) -> Result<Box<dyn UninitializedArmProbe + 'probe>, (Self, DebugProbeError)> {
         if !self.attached {
             Err((self, DebugProbeError::NotAttached))
         } else {
@@ -422,6 +395,10 @@ impl Probe {
     pub fn get_swo_interface_mut(&mut self) -> Option<&mut dyn SwoAccess> {
         self.inner.get_swo_interface_mut()
     }
+
+    pub fn try_as_dap_probe(&mut self) -> Option<&mut dyn DapProbe> {
+        self.inner.try_as_dap_probe()
+    }
 }
 
 pub trait DebugProbe: Send + fmt::Debug {
@@ -485,7 +462,8 @@ pub trait DebugProbe: Send + fmt::Debug {
     /// probe actually supports this, call [DebugProbe::has_arm_interface] first.
     fn try_get_arm_interface<'probe>(
         self: Box<Self>,
-    ) -> Result<Box<dyn ArmProbeInterface + 'probe>, (Box<dyn DebugProbe>, DebugProbeError)> {
+    ) -> Result<Box<dyn UninitializedArmProbe + 'probe>, (Box<dyn DebugProbe>, DebugProbeError)>
+    {
         Err((
             self.into_probe(),
             DebugProbeError::InterfaceNotAvailable("ARM"),
@@ -518,6 +496,10 @@ pub trait DebugProbe: Send + fmt::Debug {
 
     fn into_probe(self: Box<Self>) -> Box<dyn DebugProbe>;
 
+    fn try_as_dap_probe(&mut self) -> Option<&mut dyn DapProbe> {
+        None
+    }
+
     /// Reads the target voltage in Volts, if possible. Returns `Ok(None)`
     /// if the probe doesn’t support reading the target voltage.
     fn get_target_voltage(&mut self) -> Result<Option<f32>, DebugProbeError> {
@@ -531,6 +513,7 @@ pub enum DebugProbeType {
     Ftdi,
     StLink,
     JLink,
+    EspJtag,
 }
 
 #[derive(Clone)]
@@ -540,6 +523,10 @@ pub struct DebugProbeInfo {
     pub product_id: u16,
     pub serial_number: Option<String>,
     pub probe_type: DebugProbeType,
+
+    /// USB HID interface which should be used.
+    /// Necessary for composite HID devices.
+    pub hid_interface: Option<u8>,
 }
 
 impl std::fmt::Debug for DebugProbeInfo {
@@ -566,6 +553,7 @@ impl DebugProbeInfo {
         product_id: u16,
         serial_number: Option<String>,
         probe_type: DebugProbeType,
+        usb_hid_interface: Option<u8>,
     ) -> Self {
         Self {
             identifier: identifier.into(),
@@ -573,6 +561,7 @@ impl DebugProbeInfo {
             product_id,
             serial_number,
             probe_type,
+            hid_interface: usb_hid_interface,
         }
     }
 
@@ -600,7 +589,8 @@ pub enum DebugProbeSelectorParseError {
 /// let selector: probe_rs::DebugProbeSelector = "1337:1337:SERIAL".try_into().unwrap();
 /// ```
 #[derive(Debug, Clone, Serialize, Deserialize)]
-#[serde(try_from = "String")] //We need this so that serde will first converst from the string `PID:VID:<Serial>` to a struct before deserializing
+// We need this so that serde will first converst from the string `PID:VID:<Serial>` to a struct before deserializing.
+#[serde(try_from = "String")]
 pub struct DebugProbeSelector {
     pub vendor_id: u16,
     pub product_id: u16,
@@ -673,215 +663,6 @@ impl fmt::Display for DebugProbeSelector {
     }
 }
 
-#[derive(Debug)]
-pub struct FakeProbe {
-    protocol: WireProtocol,
-    speed: u32,
-}
-
-impl FakeProbe {
-    pub fn new() -> Self {
-        FakeProbe {
-            protocol: WireProtocol::Swd,
-            speed: 1000,
-        }
-    }
-}
-
-impl Default for FakeProbe {
-    fn default() -> Self {
-        FakeProbe::new()
-    }
-}
-
-impl DebugProbe for FakeProbe {
-    fn new_from_selector(
-        _selector: impl Into<DebugProbeSelector>,
-    ) -> Result<Box<Self>, DebugProbeError>
-    where
-        Self: Sized,
-    {
-        Ok(Box::new(FakeProbe::new()))
-    }
-
-    /// Get human readable name for the probe
-    fn get_name(&self) -> &str {
-        "Mock probe for testing"
-    }
-
-    fn speed(&self) -> u32 {
-        self.speed
-    }
-
-    fn set_speed(&mut self, speed_khz: u32) -> Result<u32, DebugProbeError> {
-        self.speed = speed_khz;
-
-        Ok(speed_khz)
-    }
-
-    fn attach(&mut self) -> Result<(), DebugProbeError> {
-        Ok(())
-    }
-
-    fn select_protocol(&mut self, protocol: WireProtocol) -> Result<(), DebugProbeError> {
-        self.protocol = protocol;
-
-        Ok(())
-    }
-
-    /// Leave debug mode
-    fn detach(&mut self) -> Result<(), DebugProbeError> {
-        Ok(())
-    }
-
-    /// Resets the target device.
-    fn target_reset(&mut self) -> Result<(), DebugProbeError> {
-        Err(DebugProbeError::CommandNotSupportedByProbe)
-    }
-
-    fn target_reset_assert(&mut self) -> Result<(), DebugProbeError> {
-        unimplemented!()
-    }
-
-    fn target_reset_deassert(&mut self) -> Result<(), DebugProbeError> {
-        unimplemented!()
-    }
-
-    fn into_probe(self: Box<Self>) -> Box<dyn DebugProbe> {
-        self
-    }
-
-    fn has_arm_interface(&self) -> bool {
-        true
-    }
-
-    fn try_get_arm_interface<'probe>(
-        self: Box<Self>,
-    ) -> Result<Box<dyn ArmProbeInterface + 'probe>, (Box<dyn DebugProbe>, DebugProbeError)> {
-        Ok(Box::new(FakeArmInterface::new(self)))
-    }
-}
-
-impl DapAccess for FakeProbe {
-    /// Reads the DAP register on the specified port and address
-    fn read_register(&mut self, _port: PortType, _addr: u8) -> Result<u32, DebugProbeError> {
-        Err(DebugProbeError::CommandNotSupportedByProbe)
-    }
-
-    /// Writes a value to the DAP register on the specified port and address
-    fn write_register(
-        &mut self,
-        _port: PortType,
-        _addr: u8,
-        _value: u32,
-    ) -> Result<(), DebugProbeError> {
-        Err(DebugProbeError::CommandNotSupportedByProbe)
-    }
-}
-
-#[derive(Debug)]
-struct FakeArmInterface {
-    probe: Box<FakeProbe>,
-
-    memory_ap: MockMemoryAp,
-}
-
-impl FakeArmInterface {
-    fn new(probe: Box<FakeProbe>) -> Self {
-        let memory_ap = MockMemoryAp::with_pattern();
-        FakeArmInterface { probe, memory_ap }
-    }
-}
-
-impl ArmProbeInterface for FakeArmInterface {
-    fn memory_interface(&mut self, access_port: MemoryAp) -> Result<Memory<'_>, Error> {
-        let ap_information = MemoryApInformation {
-            port_number: access_port.port_number(),
-            only_32bit_data_size: false,
-            debug_base_address: 0xf000_0000,
-            supports_hnonsec: false,
-        };
-
-        let memory = ADIMemoryInterface::new(&mut self.memory_ap, &ap_information)?;
-
-        Ok(Memory::new(memory, access_port))
-    }
-
-    fn ap_information(
-        &self,
-        _access_port: crate::architecture::arm::ap::GenericAp,
-    ) -> Option<&crate::architecture::arm::ApInformation> {
-        todo!()
-    }
-
-    fn num_access_ports(&self) -> usize {
-        1
-    }
-
-    fn read_from_rom_table(
-        &mut self,
-    ) -> Result<Option<crate::architecture::arm::ArmChipInfo>, Error> {
-        Ok(None)
-    }
-
-    fn close(self: Box<Self>) -> Probe {
-        Probe::from_attached_probe(self.probe)
-    }
-
-    fn target_reset_deassert(&mut self) -> Result<(), Error> {
-        todo!()
-    }
-}
-
-impl RawApAccess for FakeArmInterface {
-    type Error = DebugProbeError;
-
-    fn read_raw_ap_register(&mut self, _port_number: u8, _address: u8) -> Result<u32, Self::Error> {
-        todo!()
-    }
-
-    fn read_raw_ap_register_repeated(
-        &mut self,
-        _port: u8,
-        _address: u8,
-        _values: &mut [u32],
-    ) -> Result<(), Self::Error> {
-        todo!()
-    }
-
-    fn write_raw_ap_register(
-        &mut self,
-        _port: u8,
-        _address: u8,
-        _value: u32,
-    ) -> Result<(), Self::Error> {
-        todo!()
-    }
-
-    fn write_raw_ap_register_repeated(
-        &mut self,
-        _port: u8,
-        _address: u8,
-        _values: &[u32],
-    ) -> Result<(), Self::Error> {
-        todo!()
-    }
-}
-
-impl SwoAccess for FakeArmInterface {
-    fn enable_swo(&mut self, _config: &crate::architecture::arm::SwoConfig) -> Result<(), Error> {
-        unimplemented!()
-    }
-
-    fn disable_swo(&mut self) -> Result<(), Error> {
-        unimplemented!()
-    }
-
-    fn read_swo_timeout(&mut self, _timeout: std::time::Duration) -> Result<Vec<u8>, Error> {
-        unimplemented!()
-    }
-}
-
 /// Low-Level Access to the JTAG protocol
 ///
 /// This trait should be implemented by all probes which offer low-level access to
@@ -895,6 +676,9 @@ pub trait JTAGAccess: DebugProbe {
     /// This function configures the number of idle cycles which are inserted after each access.
     fn set_idle_cycles(&mut self, idle_cycles: u8);
 
+    /// Return the currently configured idle cycles.
+    fn get_idle_cycles(&self) -> u8;
+
     /// Write to a JTAG register
     ///
     /// This function will perform a write to the IR register, if necessary,
@@ -906,6 +690,69 @@ pub trait JTAGAccess: DebugProbe {
         data: &[u8],
         len: u32,
     ) -> Result<Vec<u8>, DebugProbeError>;
+
+    fn write_register_batch(
+        &mut self,
+        writes: &[JtagWriteCommand],
+    ) -> Result<Vec<CommandResult>, BatchExecutionError> {
+        let mut results = Vec::new();
+
+        for write in writes {
+            match self
+                .write_register(write.address, &write.data, write.len)
+                .and_then(|response| (write.transform)(response))
+            {
+                Ok(res) => results.push(res),
+                Err(e) => return Err(BatchExecutionError::new(e, results.clone())),
+            }
+        }
+
+        Ok(results)
+    }
+}
+
+pub type DeferredResultIndex = usize;
+
+#[derive(Debug, Clone)]
+pub struct JtagWriteCommand {
+    pub address: u32,
+    pub data: Vec<u8>,
+    pub len: u32,
+    pub transform: fn(Vec<u8>) -> Result<CommandResult, DebugProbeError>,
+}
+
+#[derive(thiserror::Error, Debug)]
+pub struct BatchExecutionError {
+    #[source]
+    pub error: DebugProbeError,
+    pub results: Vec<CommandResult>,
+}
+
+impl BatchExecutionError {
+    pub fn new(error: DebugProbeError, results: Vec<CommandResult>) -> BatchExecutionError {
+        BatchExecutionError { error, results }
+    }
+}
+
+impl std::fmt::Display for BatchExecutionError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(
+            f,
+            "Error cause was {}. Successful command count {}",
+            self.error,
+            self.results.len()
+        )
+    }
+}
+
+/// Results generated by `JtagCommand`s
+#[derive(Debug, Clone)]
+pub enum CommandResult {
+    None,
+    U8(u8),
+    U16(u16),
+    U32(u32),
+    VecU8(Vec<u8>),
 }
 
 #[derive(PartialEq, Debug, Copy, Clone)]
