@@ -1,4 +1,4 @@
-use super::super::{Category, Request, Response, SendError, Status};
+use super::super::{CommandId, Request, SendError, Status};
 
 /// The DAP_TransferConfigure Command sets parameters for DAP_Transfer and DAP_TransferBlock.
 #[derive(Debug)]
@@ -12,27 +12,27 @@ pub struct ConfigureRequest {
 }
 
 impl Request for ConfigureRequest {
-    const CATEGORY: Category = Category(0x04);
+    const COMMAND_ID: CommandId = CommandId::TransferConfigure;
 
-    fn to_bytes(&self, buffer: &mut [u8], offset: usize) -> Result<usize, SendError> {
+    type Response = ConfigureResponse;
+
+    fn to_bytes(&self, buffer: &mut [u8]) -> Result<usize, SendError> {
         use scroll::{Pwrite, LE};
 
-        buffer[offset] = self.idle_cycles;
+        buffer[0] = self.idle_cycles;
         buffer
-            .pwrite_with(self.wait_retry, offset + 1, LE)
-            .map_err(|_| SendError::Bug)?;
+            .pwrite_with(self.wait_retry, 1, LE)
+            .expect("This is a bug. Please report it.");
         buffer
-            .pwrite_with(self.match_retry, offset + 3, LE)
-            .map_err(|_| SendError::Bug)?;
+            .pwrite_with(self.match_retry, 3, LE)
+            .expect("This is a bug. Please report it.");
         Ok(5)
     }
+
+    fn from_bytes(&self, buffer: &[u8]) -> Result<Self::Response, SendError> {
+        Ok(ConfigureResponse(Status::from_byte(buffer[0])?))
+    }
 }
 
 #[derive(Debug)]
 pub struct ConfigureResponse(pub(crate) Status);
-
-impl Response for ConfigureResponse {
-    fn from_bytes(buffer: &[u8], offset: usize) -> Result<Self, SendError> {
-        Ok(ConfigureResponse(Status::from_byte(buffer[offset])?))
-    }
-}
