@@ -3,20 +3,25 @@ pub mod tools;
 mod usb_interface;
 
 use self::usb_interface::{StLinkUsb, StLinkUsbDevice};
-use super::{DebugProbe, DebugProbeError, PortType, ProbeCreationError, WireProtocol};
+use super::{DebugProbe, DebugProbeError, ProbeCreationError, WireProtocol};
 use crate::{
     architecture::arm::{
-        ap::{valid_access_ports, AccessPort, ApAccess, ApClass, MemoryAp, RawApAccess, IDR},
-        communication_interface::{ArmCommunicationInterfaceState, ArmProbeInterface},
-        dp::{DebugPortError, DebugPortVersion, DpBankSel, RawDpAccess},
+        ap::{valid_access_ports, AccessPort, ApAccess, ApClass, MemoryAp, IDR},
+        communication_interface::{
+            ArmProbeInterface, Initialized, Register, SwdSequence, UninitializedArmProbe,
+        },
+        dp::DPIDR,
         memory::{adi_v5_memory_interface::ArmProbe, Component},
-        ApInformation, ArmChipInfo, SwoAccess, SwoConfig, SwoMode,
+        sequences::ArmDebugSequence,
+        ApAddress, ApInformation, ArmChipInfo, DapAccess, DpAddress, Pins, SwoAccess, SwoConfig,
+        SwoMode,
     },
     DebugProbeSelector, Error as ProbeRsError, Memory, Probe,
 };
+use anyhow::anyhow;
 use constants::{commands, JTagFrequencyToDivider, Mode, Status, SwdFrequencyToDelayCount};
 use scroll::{Pread, Pwrite, BE, LE};
-use std::{cmp::Ordering, convert::TryInto, time::Duration};
+use std::{cmp::Ordering, convert::TryInto, sync::Arc, time::Duration};
 use usb_interface::TIMEOUT;
 
 /// Maximum length of 32 bit reads in bytes.
@@ -30,6 +35,8 @@ const STLINK_MAX_READ_LEN: usize = 6144;
 /// is also a multiple of 4.
 const STLINK_MAX_WRITE_LEN: usize = 0xFFFC;
 
+const DP_PORT: u16 = 0xFFFF;
+
 #[derive(Debug)]
 pub struct StLink<D: StLinkUsb> {
     device: D,
@@ -263,11 +270,9 @@ impl DebugProbe for StLink<StLinkUsbDevice> {
 
     fn try_get_arm_interface<'probe>(
         self: Box<Self>,
-    ) -> Result<Box<dyn ArmProbeInterface + 'probe>, (Box<dyn DebugProbe>, DebugProbeError)> {
-        match StlinkArmDebug::new(self) {
-            Ok(interface) => Ok(Box::new(interface)),
-            Err((probe, err)) => Err((probe.into_probe(), err)),
-        }
+    ) -> Result<Box<dyn UninitializedArmProbe + 'probe>, (Box<dyn DebugProbe>, DebugProbeError)>
+    {
+        Ok(Box::new(UninitializedStLink { probe: self }))
     }
 
     fn get_target_voltage(&mut self) -> Result<Option<f32>, DebugProbeError> {
@@ -298,6 +303,40 @@ impl<D: StLinkUsb> Drop for StLink<D> {
     }
 }
 
+impl StLink<StLinkUsbDevice> {
+    fn swj_pins(
+        &mut self,
+        pin_out: u32,
+        pin_select: u32,
+        pin_wait: u32,
+    ) -> Result<u32, ProbeRsError> {
+        let mut nreset = Pins(0);
+        nreset.set_nreset(true);
+        let nreset_mask = nreset.0 as u32;
+
+        // If only the reset pin is selected we perform the reset.
+        // If something else is selected return an error as this is not supported on ST-Links.
+        if pin_select == nreset_mask {
+            if Pins(pin_out as u8).nreset() {
+                self.target_reset_deassert()?;
+            } else {
+                self.target_reset_assert()?;
+            }
+
+            // Normally this would be the timeout we pass to the probe to settle the pins.
+            // The ST-Link is not capable of this, so we just wait for this time on the host
+            // and assume it has settled until then.
+            std::thread::sleep(Duration::from_micros(pin_wait as u64));
+
+            // We signal that we cannot read the pin state.
+            Ok(0xFFFF_FFFF)
+        } else {
+            // This is not supported for ST-Links, unfortunately.
+            Err(DebugProbeError::CommandNotSupportedByProbe("swj_pins").into())
+        }
+    }
+}
+
 impl<D: StLinkUsb> StLink<D> {
     /// Maximum number of bytes to send or receive for 32- and 16- bit transfers.
     ///
@@ -490,7 +529,9 @@ impl<D: StLinkUsb> StLink<D> {
         frequency_khz: u32,
     ) -> Result<(), DebugProbeError> {
         if self.hw_version != 3 {
-            return Err(DebugProbeError::CommandNotSupportedByProbe);
+            return Err(DebugProbeError::CommandNotSupportedByProbe(
+                "set_communication_frequency",
+            ));
         }
 
         let cmd_proto = match protocol {
@@ -511,7 +552,9 @@ impl<D: StLinkUsb> StLink<D> {
         protocol: WireProtocol,
     ) -> Result<(Vec<u32>, u32), DebugProbeError> {
         if self.hw_version != 3 {
-            return Err(DebugProbeError::CommandNotSupportedByProbe);
+            return Err(DebugProbeError::CommandNotSupportedByProbe(
+                "get_communication_frequencies",
+            ));
         }
 
         let cmd_proto = match protocol {
@@ -570,7 +613,7 @@ impl<D: StLinkUsb> StLink<D> {
     fn open_ap(&mut self, apsel: u8) -> Result<(), DebugProbeError> {
         // Ensure this command is actually supported
         if self.hw_version < 3 && self.jtag_version < Self::MIN_JTAG_VERSION_MULTI_AP {
-            return Err(DebugProbeError::CommandNotSupportedByProbe);
+            return Err(DebugProbeError::CommandNotSupportedByProbe("open_ap"));
         }
 
         let mut buf = [0; 2];
@@ -590,7 +633,7 @@ impl<D: StLinkUsb> StLink<D> {
     fn _close_ap(&mut self, apsel: u8) -> Result<(), DebugProbeError> {
         // Ensure this command is actually supported
         if self.hw_version < 3 && self.jtag_version < Self::MIN_JTAG_VERSION_MULTI_AP {
-            return Err(DebugProbeError::CommandNotSupportedByProbe);
+            return Err(DebugProbeError::CommandNotSupportedByProbe("close_ap"));
         }
 
         let mut buf = [0; 2];
@@ -704,63 +747,58 @@ impl<D: StLinkUsb> StLink<D> {
     }
 
     /// Reads the DAP register on the specified port and address.
-    fn read_register(&mut self, port: PortType, addr: u8) -> Result<u32, DebugProbeError> {
-        if (addr & 0xf0) == 0 || port != PortType::DebugPort {
-            if let PortType::AccessPort(port_number) = port {
-                self.select_ap(port_number as u8)?;
-            }
-
-            let port = u16::from(port).to_le_bytes();
-
-            let cmd = &[
-                commands::JTAG_COMMAND,
-                commands::JTAG_READ_DAP_REG,
-                port[0],
-                port[1],
-                addr,
-                0, // Maximum address for DAP registers is 0xFC
-            ];
-            let mut buf = [0; 8];
-            self.send_jtag_command(cmd, &[], &mut buf, TIMEOUT)?;
-            // Unwrap is ok!
-            Ok((&buf[4..8]).pread_with(0, LE).unwrap())
-        } else {
-            Err(StlinkError::BlanksNotAllowedOnDPRegister.into())
+    fn read_register(&mut self, port: u16, addr: u8) -> Result<u32, DebugProbeError> {
+        if port == DP_PORT && addr & 0xf0 != 0 {
+            return Err(StlinkError::BanksNotAllowedOnDPRegister.into());
         }
+
+        if port != DP_PORT {
+            self.select_ap(port as u8)?;
+        }
+
+        let port = port.to_le_bytes();
+
+        let cmd = &[
+            commands::JTAG_COMMAND,
+            commands::JTAG_READ_DAP_REG,
+            port[0],
+            port[1],
+            addr,
+            0, // Maximum address for DAP registers is 0xFC
+        ];
+        let mut buf = [0; 8];
+        self.send_jtag_command(cmd, &[], &mut buf, TIMEOUT)?;
+        // Unwrap is ok!
+        Ok((&buf[4..8]).pread_with(0, LE).unwrap())
     }
 
     /// Writes a value to the DAP register on the specified port and address.
-    fn write_register(
-        &mut self,
-        port: PortType,
-        addr: u8,
-        value: u32,
-    ) -> Result<(), DebugProbeError> {
-        if (addr & 0xf0) == 0 || port != PortType::DebugPort {
-            if let PortType::AccessPort(port_number) = port {
-                self.select_ap(port_number as u8)?;
-            }
-
-            let port = u16::from(port).to_le_bytes();
-            let bytes = value.to_le_bytes();
-
-            let cmd = &[
-                commands::JTAG_COMMAND,
-                commands::JTAG_WRITE_DAP_REG,
-                port[0],
-                port[1],
-                addr,
-                0, // Maximum address for DAP registers is 0xFC
-                bytes[0],
-                bytes[1],
-                bytes[2],
-                bytes[3],
-            ];
-            let mut buf = [0; 2];
-            self.send_jtag_command(cmd, &[], &mut buf, TIMEOUT)
-        } else {
-            Err(StlinkError::BlanksNotAllowedOnDPRegister.into())
+    fn write_register(&mut self, port: u16, addr: u8, value: u32) -> Result<(), DebugProbeError> {
+        if port == DP_PORT && addr & 0xf0 != 0 {
+            return Err(StlinkError::BanksNotAllowedOnDPRegister.into());
         }
+
+        if port != DP_PORT {
+            self.select_ap(port as u8)?;
+        }
+
+        let port = port.to_le_bytes();
+        let bytes = value.to_le_bytes();
+
+        let cmd = &[
+            commands::JTAG_COMMAND,
+            commands::JTAG_WRITE_DAP_REG,
+            port[0],
+            port[1],
+            addr,
+            0, // Maximum address for DAP registers is 0xFC
+            bytes[0],
+            bytes[1],
+            bytes[2],
+            bytes[3],
+        ];
+        let mut buf = [0; 2];
+        self.send_jtag_command(cmd, &[], &mut buf, TIMEOUT)
     }
 
     fn read_mem_32bit(
@@ -919,7 +957,7 @@ impl<D: StLinkUsb> StLink<D> {
                 lenbytes[1],
                 apsel,
             ],
-            &data,
+            data,
             &mut [],
             TIMEOUT,
         )?;
@@ -1074,8 +1112,8 @@ pub(crate) enum StlinkError {
     VoltageDivisionByZero,
     #[error("Probe is an unknown mode.")]
     UnknownMode,
-    #[error("Blank values are not allowed on DebugPort writes.")]
-    BlanksNotAllowedOnDPRegister,
+    #[error("STLink does not support accessing banked DP registers.")]
+    BanksNotAllowedOnDPRegister,
     #[error("Not enough bytes written.")]
     NotEnoughBytesWritten { is: usize, should: usize },
     #[error("Usb endpoint not found.")]
@@ -1086,6 +1124,8 @@ pub(crate) enum StlinkError {
     JTAGNotSupportedOnProbe,
     #[error("Manchester-coded SWO mode not supported")]
     ManchesterSwoNotSupported,
+    #[error("Multidrop SWD not supported")]
+    MultidropNotSupported,
     #[error("Unaligned")]
     UnalignedAddress,
 }
@@ -1102,60 +1142,124 @@ impl From<StlinkError> for ProbeCreationError {
     }
 }
 
+struct UninitializedStLink {
+    probe: Box<StLink<StLinkUsbDevice>>,
+}
+
+impl UninitializedArmProbe for UninitializedStLink {
+    // Todo: Is this possible with an uninitialized ST Link?
+    fn read_dpidr(&mut self) -> Result<u32, ProbeRsError> {
+        let result = self
+            .probe
+            .read_register(DP_PORT, DPIDR::ADDRESS)
+            .map_err(|e| ProbeRsError::Other(e.into()))?;
+
+        Ok(result)
+    }
+
+    fn initialize(
+        self: Box<Self>,
+        _sequence: Arc<dyn ArmDebugSequence>,
+    ) -> Result<Box<dyn ArmProbeInterface>, ProbeRsError> {
+        let interface = StlinkArmDebug::new(self.probe).map_err(|(_s, e)| ProbeRsError::from(e))?;
+
+        Ok(Box::new(interface))
+    }
+}
+
+impl SwdSequence for UninitializedStLink {
+    fn swj_sequence(&mut self, _bit_len: u8, _bits: u64) -> Result<(), ProbeRsError> {
+        // This is not supported for ST-Links, unfortunately.
+        Err(DebugProbeError::CommandNotSupportedByProbe("swj_sequence").into())
+    }
+
+    fn swj_pins(
+        &mut self,
+        pin_out: u32,
+        pin_select: u32,
+        pin_wait: u32,
+    ) -> Result<u32, ProbeRsError> {
+        self.probe.swj_pins(pin_out, pin_select, pin_wait)
+    }
+}
+
 #[derive(Debug)]
 struct StlinkArmDebug {
     probe: Box<StLink<StLinkUsbDevice>>,
-    state: ArmCommunicationInterfaceState,
+
+    /// Information about the APs of the target.
+    /// APs are identified by a number, starting from zero.
+    pub ap_information: Vec<ApInformation>,
 }
 
 impl StlinkArmDebug {
     fn new(
         probe: Box<StLink<StLinkUsbDevice>>,
     ) -> Result<Self, (Box<StLink<StLinkUsbDevice>>, DebugProbeError)> {
-        let state = ArmCommunicationInterfaceState::new();
-
         // Determine the number and type of available APs.
 
-        let mut interface = Self { probe, state };
+        let mut interface = Self {
+            probe,
+            ap_information: Vec::new(),
+        };
 
-        for ap in valid_access_ports(&mut interface) {
+        for ap in valid_access_ports(&mut interface, DpAddress::Default) {
             let ap_state = match ApInformation::read_from_target(&mut interface, ap) {
                 Ok(state) => state,
                 Err(e) => return Err((interface.probe, e)),
             };
 
-            log::debug!("AP {}: {:?}", ap.port_number(), ap_state);
+            log::debug!("AP {:#x?}: {:?}", ap.ap_address(), ap_state);
 
-            interface.state.ap_information.push(ap_state);
+            interface.ap_information.push(ap_state);
         }
 
         Ok(interface)
     }
 }
 
-impl RawDpAccess for StlinkArmDebug {
-    fn read_raw_dp_register(
-        &mut self,
-        _bank: DpBankSel,
-        address: u8,
-    ) -> Result<u32, DebugPortError> {
-        let result = self.probe.read_register(PortType::DebugPort, address)?;
+impl DapAccess for StlinkArmDebug {
+    fn read_raw_dp_register(&mut self, dp: DpAddress, address: u8) -> Result<u32, DebugProbeError> {
+        if dp != DpAddress::Default {
+            return Err(StlinkError::MultidropNotSupported.into());
+        }
+        let result = self.probe.read_register(DP_PORT, address)?;
         Ok(result)
     }
 
     fn write_raw_dp_register(
         &mut self,
-        _bank: DpBankSel,
+        dp: DpAddress,
         address: u8,
         value: u32,
-    ) -> Result<(), DebugPortError> {
-        self.probe
-            .write_register(PortType::DebugPort, address, value)?;
+    ) -> Result<(), DebugProbeError> {
+        if dp != DpAddress::Default {
+            return Err(StlinkError::MultidropNotSupported.into());
+        }
+
+        self.probe.write_register(DP_PORT, address, value)?;
         Ok(())
     }
 
-    fn debug_port_version(&self) -> DebugPortVersion {
-        self.state.debug_port_version
+    fn read_raw_ap_register(&mut self, ap: ApAddress, address: u8) -> Result<u32, DebugProbeError> {
+        if ap.dp != DpAddress::Default {
+            return Err(StlinkError::MultidropNotSupported.into());
+        }
+
+        self.probe.read_register(ap.ap as u16, address)
+    }
+
+    fn write_raw_ap_register(
+        &mut self,
+        ap: ApAddress,
+        address: u8,
+        value: u32,
+    ) -> Result<(), DebugProbeError> {
+        if ap.dp != DpAddress::Default {
+            return Err(StlinkError::MultidropNotSupported.into());
+        }
+
+        self.probe.write_register(ap.ap as u16, address, value)
     }
 }
 
@@ -1167,18 +1271,30 @@ impl<'probe> ArmProbeInterface for StlinkArmDebug {
     }
 
     fn ap_information(
-        &self,
+        &mut self,
         access_port: crate::architecture::arm::ap::GenericAp,
-    ) -> Option<&crate::architecture::arm::communication_interface::ApInformation> {
-        self.state
-            .ap_information
-            .get(access_port.port_number() as usize)
+    ) -> Result<&crate::architecture::arm::communication_interface::ApInformation, ProbeRsError>
+    {
+        let addr = access_port.ap_address();
+        if addr.dp != DpAddress::Default {
+            return Err(DebugProbeError::from(StlinkError::MultidropNotSupported).into());
+        }
+
+        match self.ap_information.get(addr.ap as usize) {
+            Some(res) => Ok(res),
+            None => Err(anyhow!("AP {:#x?} does not exist", addr).into()),
+        }
     }
 
     fn read_from_rom_table(
         &mut self,
+        dp: DpAddress,
     ) -> Result<Option<crate::architecture::arm::ArmChipInfo>, ProbeRsError> {
-        for access_port in valid_access_ports(self) {
+        if dp != DpAddress::Default {
+            return Err(DebugProbeError::from(StlinkError::MultidropNotSupported).into());
+        }
+
+        for access_port in valid_access_ports(self, dp) {
             let idr: IDR = self
                 .read_ap_register(access_port)
                 .map_err(ProbeRsError::Probe)?;
@@ -1210,14 +1326,12 @@ impl<'probe> ArmProbeInterface for StlinkArmDebug {
         Ok(None)
     }
 
-    fn num_access_ports(&self) -> usize {
-        self.state.ap_information.len()
-    }
+    fn num_access_ports(&mut self, dp: DpAddress) -> Result<usize, ProbeRsError> {
+        if dp != DpAddress::Default {
+            return Err(DebugProbeError::from(StlinkError::MultidropNotSupported).into());
+        }
 
-    fn target_reset_deassert(&mut self) -> Result<(), ProbeRsError> {
-        self.probe.target_reset_deassert()?;
-
-        Ok(())
+        Ok(self.ap_information.len())
     }
 
     fn close(self: Box<Self>) -> Probe {
@@ -1225,22 +1339,19 @@ impl<'probe> ArmProbeInterface for StlinkArmDebug {
     }
 }
 
-impl RawApAccess for StlinkArmDebug {
-    type Error = DebugProbeError;
-
-    fn read_raw_ap_register(&mut self, port: u8, address: u8) -> Result<u32, Self::Error> {
-        self.probe
-            .read_register(PortType::AccessPort(port as u16), address)
+impl SwdSequence for StlinkArmDebug {
+    fn swj_sequence(&mut self, _bit_len: u8, _bits: u64) -> Result<(), ProbeRsError> {
+        // This is not supported for ST-Links, unfortunately.
+        Err(DebugProbeError::CommandNotSupportedByProbe("swj_seqeunce").into())
     }
 
-    fn write_raw_ap_register(
+    fn swj_pins(
         &mut self,
-        port: u8,
-        address: u8,
-        value: u32,
-    ) -> Result<(), Self::Error> {
-        self.probe
-            .write_register(PortType::AccessPort(port as u16), address, value)
+        pin_out: u32,
+        pin_select: u32,
+        pin_wait: u32,
+    ) -> Result<u32, ProbeRsError> {
+        self.probe.swj_pins(pin_out, pin_select, pin_wait)
     }
 }
 
@@ -1263,6 +1374,21 @@ struct StLinkMemoryInterface<'probe> {
     probe: &'probe mut StlinkArmDebug,
 }
 
+impl SwdSequence for StLinkMemoryInterface<'_> {
+    fn swj_sequence(&mut self, bit_len: u8, bits: u64) -> Result<(), ProbeRsError> {
+        self.probe.swj_sequence(bit_len, bits)
+    }
+
+    fn swj_pins(
+        &mut self,
+        pin_out: u32,
+        pin_select: u32,
+        pin_wait: u32,
+    ) -> Result<u32, ProbeRsError> {
+        self.probe.swj_pins(pin_out, pin_select, pin_wait)
+    }
+}
+
 impl ArmProbe for StLinkMemoryInterface<'_> {
     fn read_32(
         &mut self,
@@ -1277,7 +1403,7 @@ impl ArmProbe for StLinkMemoryInterface<'_> {
             self.probe.probe.read_mem_32bit(
                 address + (index * STLINK_MAX_READ_LEN) as u32,
                 &mut buff,
-                ap.port_number(),
+                ap.ap_address().ap,
             )?;
 
             for (index, word) in buff.chunks_exact(4).enumerate() {
@@ -1304,7 +1430,7 @@ impl ArmProbe for StLinkMemoryInterface<'_> {
             chunk.copy_from_slice(&self.probe.probe.read_mem_8bit(
                 address + (index * chunk_size) as u32,
                 chunk.len() as u16,
-                ap.port_number(),
+                ap.ap_address().ap,
             )?);
         }
 
@@ -1326,7 +1452,7 @@ impl ArmProbe for StLinkMemoryInterface<'_> {
             self.probe.probe.write_mem_32bit(
                 address + (index * STLINK_MAX_WRITE_LEN) as u32,
                 chunk,
-                ap.port_number(),
+                ap.ap_address().ap,
             )?;
         }
 
@@ -1348,7 +1474,7 @@ impl ArmProbe for StLinkMemoryInterface<'_> {
             log::trace!("write_8: small - direct 8 bit write to {:08x}", address);
             self.probe
                 .probe
-                .write_mem_8bit(address, data, ap.port_number())?;
+                .write_mem_8bit(address, data, ap.ap_address().ap)?;
         } else {
             // Handle unaligned data in the beginning.
             let bytes_beginning = if address % 4 == 0 {
@@ -1368,7 +1494,7 @@ impl ArmProbe for StLinkMemoryInterface<'_> {
                 self.probe.probe.write_mem_8bit(
                     current_address,
                     &data[..bytes_beginning],
-                    ap.port_number(),
+                    ap.ap_address().ap,
                 )?;
 
                 current_address += bytes_beginning as u32;
@@ -1392,7 +1518,7 @@ impl ArmProbe for StLinkMemoryInterface<'_> {
                 self.probe.probe.write_mem_32bit(
                     current_address + (index * STLINK_MAX_WRITE_LEN) as u32,
                     chunk,
-                    ap.port_number(),
+                    ap.ap_address().ap,
                 )?;
             }
 
@@ -1409,7 +1535,7 @@ impl ArmProbe for StLinkMemoryInterface<'_> {
                 self.probe.probe.write_mem_8bit(
                     current_address,
                     remaining_bytes,
-                    ap.port_number(),
+                    ap.ap_address().ap,
                 )?;
             }
         }
@@ -1442,6 +1568,15 @@ impl ArmProbe for StLinkMemoryInterface<'_> {
 
         Ok(())
     }
+
+    fn get_arm_communication_interface(
+        &mut self,
+    ) -> Result<&mut crate::architecture::arm::ArmCommunicationInterface<Initialized>, ProbeRsError>
+    {
+        Err(ProbeRsError::Probe(DebugProbeError::NotImplemented(
+            "ST-Links do not support raw SWD access.",
+        )))
+    }
 }
 
 #[cfg(test)]
